/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 * License:  GNU General Public
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"
#include "stringUtil.h"

Args *getArgs(int argc, char *argv[]){
  Args *args;
  char c;
  int i;
  char list[256];
  char *optString = "t:f:g:s:phmca";

  args = (Args *)emalloc(sizeof(Args));

  args->h = 0;
  args->m = 0;
  args->e = 0;
  args->s = 0;
  args->f = 0;
  args->a = 0;
  args->p = 0;
  args->g = 0.5;
  args->c = 0;
  args->t = NULL;
  for(i=0;i<256;i++)
    list[i] = 0;
  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
      case 'f':                            /* output file format */
	args->f = atoi(optarg);
	if(!list[(int)c]){
	  list[(int)c] = 1;
	}else
	  printf("Warning: -%c multiply defined\n",c);
	break;
    case 't':
      args->t = optarg;
      break;
      case 's':                            /* seed for random number generator */
	args->s = atoi(optarg);
	if(!list[(int)c]){
	  list[(int)c] = 1;
	}else
	  printf("Warning: -%c multiply defined\n",c);
	break;
      case 'g':                            /* gc content */
	args->g = atof(optarg);
	if(!list[(int)c]){
	  list[(int)c] = 1;
	}else
	  printf("Warning: -%c multiply defined\n",c);
	if(args->g < 0 || args->g > 1){
	  printf("ERROR [ms2dna]: G/C-content should >= 0 and <= 1\n");
	  args->e = 1;
	}
	break;
      case 'm':                           /* move repeat mutation to nearest neighbor */
	args->m = 1;
	break;
    case 'a':                           /* input is generated via macs | msformatter */
	args->a = 1;
	break;
      case 'p':                           /* print program information */
	args->p = 1;
	break;
    case 'c':                             /* mutate third codon position only */
      args->c = 1;
      break;
      default:
	args->e = 1;
	return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  return args;
}

void printUsage(char *version){
  printf("ms2dna version %s, copyright (c) 2007-2009 Bernhard Haubold & Peter Pfaffelhuber\n", version);
  printf("purpose: convert output of ms (Hudson, 2002) to DNA sequence data\n");
  printf("usage: ms2dna [options] [fileNames]\n");
  printf("options:\n");
  printf("\t[-f <NUM> output file format NUM\n");
  printf("\t    0: FASTA (default);\n");
  printf("\t    1: pgf]\n");
  printf("\t[-t FILE template sequence is read from FILE; default: random template]\n");
  printf("\t[-g <NUM> G/C-content; default: NUM=0.5]\n");
  printf("\t[-s <NUM> seed for random number generator is NUM;\n");
  printf("\t    default: randomSeed.dat or system clock]\n");
  printf("\t[-a input is generated via 'macs [options] 2> /dev/null | msformatter'; default: ms]\n");
  printf("\t[-m move repeat mutation to nearest neighbor; default: new random position]\n");
  printf("\t[-c mutate only third position to simulate codon data]\n");
  printf("\t[-h print this help message]\n");
  printf("\t[-p print program information]\n");
}

void printSplash(char *version){
  printf("********************************************\n");
  printf("*  ms2dna %s - Convert ms Output to DNA  *\n",version);
  printf("*                    by                    *\n");
  printf("*   Bernhard Haubold & Peter Pfaffelhuber  *\n");
  printf("*------------------------------------------*\n");
  printf("* REFERENCES                               *\n");
  printf("* 1) ms                                    *\n");
  printf("* Hudson, R. R. (2002). Generating samples *\n");
  printf("* under a Wright-Fisher neutral model of   *\n");
  printf("* genetic variation. Bioinformatics, 18:   *\n");
  printf("* 337-338.                                 *\n");
  printf("* 2) macs                                  *\n");
  printf("* Chen, G. K., Marjoram, P., Wall, J. D.   *\n");
  printf("* (2009). Genome Research, 19:136-142.     *\n");
  printf("*                                          *\n");  
  printf("* CONTACT                                  *\n");
  printf("* Code maintained by Bernhard Haubold,     *\n");
  printf("* haubold@evolbio.mpg.de                   *\n");
  printf("*                                          *\n");
  printf("* LICENSE                                  *\n");
  printf("* ms2dna is distributed under the GNU      *\n");
  printf("* General Public License. You should have  *\n");
  printf("* received a copy of the licence together  *\n");
  printf("* with this software. If not, see          *\n");
  printf("* http://www.gnu.org/licenses/.            *\n");
  printf("********************************************\n");
}
